package app.data.service;

import app.data.bean.CalculatedMatch;
import app.data.entity.Game;
import app.data.entity.Match;
import app.data.entity.Matchday;
import app.data.repository.MatchRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.vaadin.artur.helpers.CrudService;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class MatchService extends CrudService<Match, Integer> {

    private final MatchRepository repository;

    public MatchService(@Autowired MatchRepository repository) {
        this.repository = repository;
    }

    @Override
    protected MatchRepository getRepository() {
        return repository;
    }

    public List<Match> getMatches(Matchday matchday) {
        return repository.findAll().stream()
                .filter(match -> match.getMatchday().equals(matchday))
                .collect(Collectors.toList());
    }

    public Optional<Match> getFirstMatch(Matchday matchday) {
        List<Match> matches = getMatches(matchday);
        if (matches.isEmpty()) {
            return Optional.empty();
        }
        return Optional.of(matches.get(0));
    }

    public List<CalculatedMatch> getCalculatedMatches(Matchday matchday) {
        return getMatches(matchday).stream()
                .map(this::getCalculatedMatch)
                .collect(Collectors.toList());
    }

    public CalculatedMatch getCalculatedMatch(Match match) {
        return new CalculatedMatch(match, match.getPlayer1(), match.getPlayer2(), getScore1(match), getScore2(match));
    }

    private static double getScore1(Match match) {
        double score = 0;
        for (Game game : match.getGames()) {
            score += (double) ((game.getPlayer1IsWhite() ? game.getResult() : -game.getResult()) + 1) / 2;
        }
        return score;
    }

    private static double getScore2(Match match) {
        double score = 0;
        for (Game game : match.getGames()) {
            score += (double) ((game.getPlayer1IsWhite() ? -game.getResult() : game.getResult()) + 1) / 2;
        }
        return score;
    }
}
