package app.gameimage;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Objects;
import java.util.Set;

public class GameImageGenerator {

    private static final int FIELD_SIZE = 150;
    private static final Set<Character> WHITE_PIECES = Set.of('B', 'K', 'N', 'P', 'Q', 'R');
    private static final Set<Character> BLACK_PIECES = Set.of('b', 'k', 'n', 'p', 'q', 'r');
    private static final Set<Character> BLANKS = Set.of('1', '2', '3', '4', '5', '6', '7', '8');
    private static final Character ROW_DELIMITER = '/';

    private GameImageGenerator() {}

    public static BufferedImage generateImage(String fen) {
        BufferedImage image = GameImageUtils.readImage("board");
        Graphics2D g = image.createGraphics();
        g.drawImage(image, 0, 0, null);

        int x = 0;
        int y = 0;
        for (Character c : fen.split(" ")[0].toCharArray()) {
            if (isPiece(c)) {
                g.drawImage(GameImageUtils.readImage(getFilename(c)), x * FIELD_SIZE, y * FIELD_SIZE, null);
                x += 1;
                continue;
            }
            if (isRowDelimiter(c)) {
                x = 0;
                y += 1;
                continue;
            }
            if (isBlank(c)) {
                x += Character.getNumericValue(c);
                continue;
            }
            throw new IllegalArgumentException("FEN is invalid!");
        }
        g.dispose();
        return image;
    }

    private static boolean isBlank(Character c) {
        return BLANKS.contains(c);
    }

    private static boolean isRowDelimiter(Character c) {
        return Objects.equals(c, ROW_DELIMITER);
    }

    private static boolean isPiece(Character c) {
        return isWhite(c) || isBlack(c);
    }

    private static boolean isWhite(Character c) {
        return WHITE_PIECES.contains(c);
    }

    private static boolean isBlack(Character c) {
        return BLACK_PIECES.contains(c);
    }

    private static String getFilename(Character piece) {
        if (isWhite(piece)) {
            return "w" + piece.toString().toLowerCase();
        }
        if (isBlack(piece)) {
            return "b" + piece.toString();
        }
        throw new IllegalArgumentException("Character must represent a piece!");
    }
}
