package app.gameimage;

import app.data.chesscom.ChessComGame;
import app.data.entity.Game;
import app.data.service.ChessComService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Optional;

@Service
public class GameImageService {

    private final ChessComService chessComService;

    public GameImageService(@Autowired ChessComService chessComService) {
        this.chessComService = chessComService;
    }


    public boolean hasImage(Game game) {
        String path = GameImageUtils.IMAGE_DEST_PATH_SERVER + getGameNumber(game) + GameImageUtils.EXTENSION;
        File f = new File(path);
        return f.isFile();
    }

    public boolean createImageIfNotPresent(Game game) {
        if (hasImage(game)) return true;
        Optional<String> optionalFen = getFen(game);

        if (optionalFen.isPresent()) {
            GameImageUtils.writeImage(GameImageGenerator.generateImage(optionalFen.get()), getGameNumber(game));
            return true;
        }
        return false;
    }

    private String getGameNumber(Game game) {
        String[] url = game.getGameInfo().getChessComId().split("/");
        return url[url.length - 1];
    }

    private Optional<String> getFen(Game game) {
        if (game.getGameInfo().getFen() != null) return Optional.of(game.getGameInfo().getFen());
        return chessComService.getChessComGame(game).map(ChessComGame::getFen);
    }
}
