package app.navigation;

import app.utils.EntityStringUtils;
import com.vaadin.flow.router.WildcardParameter;
import org.springframework.lang.NonNull;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@SuppressWarnings("OptionalUsedAsFieldOrParameterType")
class NavigationUtils {
    private static final String EDIT = "edit";

    private NavigationUtils() {
    }

    @NonNull
    static Map<NavigationLevel, Optional<String>> getParameterMap(@WildcardParameter String param) {
        Map<NavigationLevel, Optional<String>> map = new HashMap<>();
        String[] params = param.split("/");
        if (params.length >= 1 && !params[0].equals(EDIT)) map.put(NavigationLevel.SEASON, Optional.of(params[0]));
        if (params.length >= 2 && !params[1].equals(EDIT)) map.put(NavigationLevel.MATCHDAY, Optional.of(params[1]));
        if (params.length >= 3 && !params[2].equals(EDIT)) map.put(NavigationLevel.MATCH, Optional.of(params[2]));

        map.putIfAbsent(NavigationLevel.MATCH, Optional.empty());
        map.putIfAbsent(NavigationLevel.MATCHDAY, Optional.empty());
        map.putIfAbsent(NavigationLevel.SEASON, Optional.empty());
        return map;
    }

    static boolean editFlag(@WildcardParameter String param) {
        String[] params = param.split("/");
        return params[params.length - 1].equals(EDIT);
    }

    @NonNull
    static String getWildcardParam(boolean editFlag, String... params) {
        StringBuilder stringBuilder = new StringBuilder();
        for (String param : params) {
            if (param == null || param.equals("")) {
                break;
            }
            stringBuilder.append(param).append("/");
        }
        if (editFlag) stringBuilder.append(EDIT).append("/");
        return stringBuilder.toString();
    }

    static <T> Optional<T> getObjectFromParam(List<T> objectList, Optional<String> param) {
        if (param.isEmpty()) {
            return Optional.empty();
        }
        return objectList.stream()
                .filter(object -> EntityStringUtils.getObjectStringForURL(object).equals(param.get()))
                .findFirst();
    }
}
