package app.navigation.components.button;

import app.data.entity.Matchday;
import app.navigation.Navigation;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;

import java.util.Optional;

public class NextMatchdayButton extends Button {

    private final Navigation navigation;

    public NextMatchdayButton(Navigation navigation) {
        this.navigation = navigation;

        if (!navigation.matchdayEnabled())
            throw new IllegalStateException("Cannot instantiate NextMatchdayButton when Matchdays are not enabled!");

        setIcon(new Icon(VaadinIcon.ARROW_RIGHT));

        navigation.addRunnableToBeRunAfterSelection(this::configure);
    }

    private void configure() {
        Optional<Matchday> nextMatchday = getNextMatchday();
        setEnabled(nextMatchday.isPresent());
        addClickListener(event -> nextMatchday.ifPresent(matchday -> navigation.getMatchdaySelect().setValue(matchday)));
    }

    private Optional<Matchday> getNextMatchday() {
        int index = ButtonUtils.getMatchdayIndex(navigation);
        if (index >= 0 && index < navigation.getMatchdayList().size() - 1)
            return Optional.ofNullable(navigation.getMatchdayList().get(index + 1));
        return Optional.empty();
    }
}
