package app.views.matchday.components;

import app.data.bean.CalculatedMatch;
import app.data.entity.Matchday;
import app.navigation.Navigation;
import app.navigation.components.button.NextMatchdayButton;
import app.navigation.components.button.PrevMatchdayButton;
import app.utils.EntityStringUtils;
import app.utils.StringUtils;
import app.utils.VaadinUtils;
import app.views.navigation.interfaces.ContentConfigurable;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;

import java.util.NoSuchElementException;

public class MatchdayComponent extends Div implements ContentConfigurable {

    private final Navigation navigation;

    private final HorizontalLayout header = new HorizontalLayout();
    private final Label headerLabel = new Label();
    private final Grid<CalculatedMatch> grid = new Grid<>();

    public MatchdayComponent(Navigation navigation) {
        this.navigation = navigation;

        addClassName("wrapper");
        VerticalLayout inner = new VerticalLayout();
        add(inner);
        inner.add(header, grid);

        configureHeader();
        configureGrid();
    }

    private void configureHeader() {
        header.add(new PrevMatchdayButton(this.navigation), headerLabel, new NextMatchdayButton(this.navigation));
        header.setWidthFull();

        headerLabel.addClassName("matchday-header-label");
    }


    private void configureGrid() {
        Label headerPlayer1 = new Label("Player 1");
        headerPlayer1.addClassName("column_header");
        Label headerPlayer2 = new Label("Player 2");
        headerPlayer2.addClassName("column_header");
        Label headerResult = new Label("Result");
        headerResult.addClassName("column_header");

        grid.addColumn(VaadinUtils.getPlayerRenderer(CalculatedMatch::getPlayer1))
                .setHeader(headerPlayer1)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em")
                .setFlexGrow(1);

        grid.addColumn(calculatedMatch -> "vs.")
                .setHeader("vs.")
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn(VaadinUtils.getPlayerRenderer(CalculatedMatch::getPlayer2))
                .setHeader(headerPlayer2)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em")
                .setFlexGrow(1);

        grid.addColumn(this::getResultString)
                .setHeader(headerResult)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addComponentColumn(this::createButton)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.setWidth("38em"); // TODO: find a way to set this dynamically based on column widths
        grid.setHeightByRows(true);

        grid.addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);
    }

    private Button createButton(CalculatedMatch match) {
        Button button = new Button();
        button.addThemeVariants(ButtonVariant.LUMO_TERTIARY_INLINE);

        String seasonAndMatchdayParam = navigation.getWildcardParam();
        String matchParam = EntityStringUtils.getMatchStringForURL(match.getMatch());
        String targetWildcardParam = String.format("match/%s%s/", seasonAndMatchdayParam, matchParam);

        if (match.getScore1() == 0 && match.getScore2() == 0) {
            button.setIcon(VaadinIcon.PENCIL.create());
            button.addClickListener(event -> UI.getCurrent().navigate(targetWildcardParam + "edit"));
            return button;
        }

        button.setIcon(VaadinIcon.EYE.create());
        button.addClickListener(event -> UI.getCurrent().navigate(targetWildcardParam));
        return button;
    }

    private String getResultString(CalculatedMatch match) {
        return StringUtils.getResultString(":", match.getScore1(), match.getScore2());
    }

    @Override
    public void configureContent() {
        try {
            Matchday matchday = navigation.getSelectedMatchday().orElseThrow();
            headerLabel.setText(String.format("Matchday %s", EntityStringUtils.getMatchdayString(matchday))); // TODO: add dates
            grid.setItems(navigation.getMatchService().getCalculatedMatches(matchday));
        } catch (NoSuchElementException e) {
            removeAll();
            add(navigation.getValidationLabel());
        }
    }
}
