package app.views.navigation;

import app.navigation.Navigation;
import app.navigation.components.NavigationHeader;
import app.navigation.NavigationLevel;
import app.navigation.NavigationService;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.router.BeforeEvent;
import com.vaadin.flow.router.HasUrlParameter;
import com.vaadin.flow.router.WildcardParameter;
import org.springframework.beans.factory.annotation.Autowired;

public abstract class NavigationViewBase extends VerticalLayout implements HasUrlParameter<String> {

    protected final Navigation navigation;
    protected final NavigationHeader navigationHeader;

    protected NavigationViewBase(@Autowired NavigationService navigationService,
                                 String route,
                                 NavigationLevel navigationLevel) {
        this.navigation = navigationService.getNewNavigation();
        navigation.setRoute(route);
        navigation.setNavigationLevel(navigationLevel);

        this.navigationHeader = new NavigationHeader(navigation);

        configure();
    }

    private void configure() {
        add(navigationHeader);
        addClassName("content");
        setAlignItems(FlexComponent.Alignment.CENTER);

        navigation.addRunnableToBeRunAfterSelection(this::configureContent);
    }

    /////////////
    // CONTENT //
    /////////////

    protected abstract void configureContent();

    //////////////
    // OVERRIDE //
    //////////////

    @Override
    public void removeAll() {
        super.removeAll();
        add(navigationHeader);
    }

    @Override
    public void setParameter(BeforeEvent event, @WildcardParameter String param) {
        navigation.setParameter(event, param);
    }
}
