package app.views.table.components;

import app.data.bean.PlayerForTable;
import app.data.entity.Matchday;
import app.data.service.PlayerService;
import app.navigation.Navigation;
import app.utils.StringUtils;
import app.utils.VaadinUtils;
import app.views.navigation.interfaces.ContentConfigurable;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Label;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.NoSuchElementException;

public class TableComponent extends Div implements ContentConfigurable {

    private final Navigation navigation;

    private final PlayerService playerService;

    private final Grid<PlayerForTable> grid = new Grid<>();

    public TableComponent(Navigation navigation, @Autowired PlayerService playerService) {
        this.navigation = navigation;
        this.playerService = playerService;

        addClassName("grid-wrapper");
        add(grid);

        configureGrid();
    }

    private void configureGrid() {
        // TODO: add diff to last matchday
        // TODO: rounded corners for rows?

        Label headerPlace = new Label("Place");
        headerPlace.addClassName("important-table-column-header");

        Label headerPlayer = new Label("Player");
        headerPlayer.addClassName("important-table-column-header");

        Label headerMatchesPlayed = new Label("Played");

        Label headerMatchPoints = new Label("Points");
        headerMatchPoints.addClassName("important-table-column-header");

        Label headerWon = new Label("W");

        Label headerDrawn = new Label("D");

        Label headerLost = new Label("L");

        Label headerGames = new Label("Games");

        Label headerDiff = new Label("Diff");

        grid.addColumn(VaadinUtils.getBoldStringRenderer(PlayerForTable::getPlaceString))
                .setHeader(headerPlace)
                .setTextAlign(ColumnTextAlign.CENTER)
//                .setClassNameGenerator(playerForTable -> "leftmost-column-cell")
                .setWidth("5em");

        grid.addColumn(VaadinUtils.getPlayerRenderer(PlayerForTable::getPlayer))
                .setHeader(headerPlayer)
                .setTextAlign(ColumnTextAlign.START)
                .setWidth("13em");

        grid.addColumn(PlayerForTable::getAmountOfMatches)
                .setHeader(headerMatchesPlayed)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("5em");

        grid.addColumn(VaadinUtils.getBoldStringRenderer(player -> String.valueOf(player.getMatchPoints())))
                .setHeader(headerMatchPoints)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addColumn(PlayerForTable::getAmountOfMatchesWon)
                .setHeader(headerWon)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn(PlayerForTable::getAmountOfMatchesDrawn)
                .setHeader(headerDrawn)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn(PlayerForTable::getAmountOfMatchesLost)
                .setHeader(headerLost)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn(this::getResultString)
                .setHeader(headerGames)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addColumn(player -> StringUtils.getSignedString(player.getGamePointDiff()))
                .setHeader(headerDiff)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("5em");

        grid.setWidth("51em");
        grid.setHeightByRows(true);

        grid.addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);
    }

    private String getResultString(PlayerForTable player) {
        return StringUtils.getResultString(":", player.getGamePointsForSelf(), player.getGamePointsForOpponents());
    }

    @Override
    public void configureContent() {
        try {
            Matchday matchday = navigation.getSelectedMatchday().orElseThrow();
            grid.setItems(playerService.getPlayersForTable(matchday));
        } catch (NoSuchElementException e) {
            removeAll();
            add(navigation.getValidationLabel());
        }
    }
}
